package mffs;

import java.util.List;
import java.util.Random;
import net.minecraft.src.AxisAlignedBB;
import net.minecraft.src.Block;
import net.minecraft.src.BlockContainer;
import net.minecraft.src.BlockGlass;
import net.minecraft.src.DamageSource;
import net.minecraft.src.Entity;
import net.minecraft.src.EntityLiving;
import net.minecraft.src.EntityPlayer;
import net.minecraft.src.IBlockAccess;
import net.minecraft.src.Item;
import net.minecraft.src.ItemStack;
import net.minecraft.src.Material;
import net.minecraft.src.ModLoader;
import net.minecraft.src.TileEntity;
import net.minecraft.src.World;
import net.minecraft.src.forge.ISpecialResistance;
import net.minecraft.src.forge.ITextureProvider;

public class BlockForceField extends BlockContainer implements ITextureProvider, ISpecialResistance {

	private boolean localFlag;
	private StringBuffer hasher = new StringBuffer();
	//private int renderblockpass;

	public BlockForceField(int i) {
		super(i, i, Material.glass);
		setHardness(100000F);
		setResistance(6000F);
		//renderblockpass = 0;
		localFlag = false;
		setTickRandomly(true);
	}
	
	@Override
	public void updateTick(World w, int x, int y, int z, Random random) {
		ForceFieldWorldMap ffworldmap = WorldMap.getForceFieldforWorld(w).getffmp(x, y, z);
		if(ffworldmap == null || ffworldmap.listsize() == 0 || !ffworldmap.ffworld_getfistactive())
			w.setBlockWithNotify(x, y, z, 0);
	}

    /**
     * Is this block (a) opaque and (b) a full 1m cube?  This determines whether or not to render the shared face of two
     * adjacent blocks and also whether the player can attach torches, redstone wire, etc to this block.
     */
	@Override
    public boolean isOpaqueCube()
    {
        return false;
    }

    /**
     * If this block doesn't render as an ordinary block it will return False (examples: signs, buttons, stairs, etc)
     */
	@Override
    public boolean renderAsNormalBlock()
    {
        return false;
    }

    /**
     * Return true if a player with SlikTouch can harvest this block directly, and not it's normal drops.
     */
	@Override
    protected boolean canSilkHarvest()
    {
        return true;
    }
	
	
	// @Override // client only override
    public int getRenderBlockPass()
    {
        return 1;
    }

    
	@Override
    public void onBlockRemoval(World world, int i, int j, int k) {

		if(!world.isRemote) {
			hasher.setLength(0);
			hasher.append(i).append("/").append(j).append("/").append(k);
			ForceFieldWorldMap ffworldmap = WorldMap.getForceFieldforWorld(world).getFFWM(hasher.toString());
			if (ffworldmap != null && ffworldmap.listsize() >= 1 && ffworldmap.ffworld_getfistactive() == true) {
				ffworldmap.ffworld_setfirstfreeospace(true);
				ffworldmap.setsync(false);

				world.setBlock(i, j, k, mod_ModularForceFieldSystem.MFFSFieldblock.blockID);

				TileEntity tileEntity = Linkgrid.getWorldMap(world).getGenerator().get(ffworldmap.ffworld_getfirstGenerator_ID());
				if (tileEntity instanceof TileEntityGeneratorCore && tileEntity != null) {
					if (ffworldmap.ffworld_getfistmode() == 1) {
						((TileEntityGeneratorCore) tileEntity).Energylost(mod_ModularForceFieldSystem.forcefieldblockcost * mod_ModularForceFieldSystem.forcefieldblockcreatemodifier);
					} else {
						((TileEntityGeneratorCore) tileEntity).Energylost(mod_ModularForceFieldSystem.forcefieldblockcost * mod_ModularForceFieldSystem.forcefieldblockcreatemodifier * mod_ModularForceFieldSystem.forcefieldblockzappermodifier);
					}

				}

			}
		}
	}

	@Override
    public AxisAlignedBB getCollisionBoundingBoxFromPool(World world, int i, int j, int k) {
		if (world.getBlockMetadata(i, j, k) == 1) {
			float f = 0.0625F;
			return AxisAlignedBB.getBoundingBoxFromPool(i + f, j + f, k + f, i + 1 - f, j + 1 - f, k + 1 - f);
		}

		return AxisAlignedBB.getBoundingBoxFromPool((float) i, j, (float) k, (float) (i + 1), (float) (j + 1), (float) (k + 1));
	}

	// @Override // client only override
    public AxisAlignedBB getSelectedBoundingBoxFromPool(World world, int i, int j, int k) {
		return AxisAlignedBB.getBoundingBoxFromPool((float) i, j, (float) k, (float) (i + 1), j + 1, (float) (k + 1));
	}

	@Override
    public void onEntityCollidedWithBlock(World world, int i, int j, int k, Entity entity) {

		if(!world.isRemote && world.getBlockMetadata(i, j, k) == 1) {
			if (entity instanceof EntityLiving) {
				entity.attackEntityFrom(DamageSource.generic, 5);
			}
		}
	}

	@Override
    public int quantityDropped(Random random) {
		return 0;
	}

	/* $if client$ */
	@Override
    public boolean shouldSideBeRendered(IBlockAccess iblockaccess, int i, int j, int k, int l) {
		if(iblockaccess.getBlockId(i, j, k) == blockID)
			return false;
		
		return super.shouldSideBeRendered(iblockaccess, i, j, k, l);
	}
	/* $endif$ */

	@Override
    public String getTextureFile() {

		return "/mffs_grafik/blocks.png";
	}

	@Override
    public boolean blockActivated(World world, int i, int j, int k, EntityPlayer entityplayer) {

		return false;
	}

	/* $if client$ */
	@Override
    public int getBlockTexture(IBlockAccess iblockaccess, int i, int j, int k, int l) {

		int meta = iblockaccess.getBlockMetadata(i, j, k);
		
		if (meta != 2) {
			return meta;
		} else {
			
			if(l < 0 || l > 5)
				return 3; // this should never happen, but it was in Thunderdark's code
			
			TileEntity te = iblockaccess.getBlockTileEntity(i, j, k);
			
			if(te == null)
				return 2;

			int camoBlockId = ((TileCamouflagedField)te).camoBlockId;

			Integer Blocktextur[] = mod_ModularForceFieldSystem.idtotextur.get(camoBlockId);

			if (Blocktextur == null) {
				return meta;
			}
			
			return Blocktextur[l];
		}
	}
	/* $endif$ */

	@Override
    public float getSpecialExplosionResistance(World world, int i, int j, int k, double d, double d1, double d2, Entity entity) {

		if(!world.isRemote) {
			hasher.setLength(0);
			hasher.append(i).append("/").append(j).append("/").append(k);
			ForceFieldWorldMap ffworldmap = WorldMap.getForceFieldforWorld(world).getFFWM(hasher.toString());
			if (ffworldmap != null && ffworldmap.listsize() >= 1 && ffworldmap.ffworld_getfistactive() == true) {
				TileEntity tileEntity = Linkgrid.getWorldMap(world).getGenerator().get(ffworldmap.ffworld_getfirstGenerator_ID());
				if (tileEntity instanceof TileEntityGeneratorCore && tileEntity != null) {
					((TileEntityGeneratorCore) tileEntity).Energylost(mod_ModularForceFieldSystem.forcefieldblockcost * mod_ModularForceFieldSystem.forcefieldblockcreatemodifier);
				}

			}
		}

		return 60000F;
	}

	// @Override // client only override
    public void randomDisplayTick(World world, int i, int j, int k, Random random) {

		if (world.getBlockMetadata(i, j, k) == 1) {

			double d = i + Math.random();
			double d1 = j + Math.random();
			double d2 = k + Math.random();

			world.spawnParticle("reddust", d, d1, d2, 0.0D, 0.0D, 0.0D);

		}
	}

	@Override
	public TileEntity getBlockEntity() {
		throw new UnsupportedOperationException("This shouldn't happen");
	}
	
	@Override
	public TileEntity getBlockEntity(int meta) {
		if(meta == 2)
			return new TileCamouflagedField();
		else
			return null;
	}

}
